Package: Chromatograms
Title: Infrastructure for Chromatographic Mass Spectrometry Data
Version: 1.2.0
Description: The Chromatograms packages defines an efficient infrastructure
   for storing and handling of chromatographic mass spectrometry data. It
   provides different implementations of *backends* to store and represent the
   data. Such backends can be optimized for small memory footprint or fast
   data access/processing. A lazy evaluation queue and chunk-wise processing
   capabilities ensure efficient analysis of also very large data sets.
Authors@R: c(person(given = "Johannes", family = "Rainer",
		    email = "Johannes.Rainer@eurac.edu",
		    role = c("aut"),
		    comment = c(ORCID = "0000-0002-6977-7147")),
		person(given = "Laurent", family = "Gatto",
		    comment = c(ORCID = "0000-0002-1520-2268"),
		    email = "laurent.gatto@uclouvain.be",
		    role = c("aut")),
	     person(given = "Philippine", family = "Louail",
		    email = "philippine.louail@outlook.com",
		    role = c("aut", "cre"),
		    comment = c(ORCID = "0009-0007-5429-6846",
		    fnd = "European Union HORIZON-MSCA-2021 project
		    Grant No. 101073062: HUMAN – Harmonising and Unifying Blood
		    Metabolic Analysis Networks")))
Depends: BiocParallel, ProtGenerics (>= 1.39.2), R (>= 4.5.0)
Imports: data.table, methods, S4Vectors, MsCoreUtils (>= 1.7.5),
        Spectra
Suggests: msdata (>= 0.19.3), roxygen2, BiocStyle (>= 2.5.19),
        testthat, knitr (>= 1.1.0), rmarkdown, mzR (>= 2.41.4),
        MsBackendMetaboLights (>= 1.3.1), pheatmap, vdiffr, IRanges,
        RColorBrewer
License: Artistic-2.0
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/RforMassSpectrometry/Chromatograms/issues
URL: https://github.com/RforMassSpectrometry/Chromatograms
biocViews: Infrastructure, Metabolomics, MassSpectrometry, Proteomics
Roxygen: list(markdown=TRUE)
RoxygenNote: 7.3.3
Collate: 'AllGenerics.R' 'ChromBackend-functions.R' 'ChromBackend.R'
        'hidden_aliases.R' 'helpers.R' 'ChromBackendMemory.R'
        'ChromBackendMzR.R' 'ChromBackendSpectra.R' 'Chromatograms.R'
        'Chromatograms-chromData.R' 'Chromatograms-peaksData.R'
        'Chromatograms-plot.R' 'Chromatograms-processingQueue.R'
git_url: https://git.bioconductor.org/packages/Chromatograms
git_branch: RELEASE_3_23
git_last_commit: afed21c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:13:27 UTC; biocbuild
Author: Johannes Rainer [aut] (ORCID: <https://orcid.org/0000-0002-6977-7147>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Philippine Louail [aut, cre] (ORCID:
    <https://orcid.org/0009-0007-5429-6846>, fnd: European Union
    HORIZON-MSCA-2021 project Grant No. 101073062: HUMAN – Harmonising
    and Unifying Blood Metabolic Analysis Networks)
Maintainer: Philippine Louail <philippine.louail@outlook.com>
