# Generated by roxygen2: do not edit by hand

export(ChromBackendMemory)
export(ChromBackendMzR)
export(ChromBackendSpectra)
export(chromSpectraIndex)
export(concatenateChromatograms)
export(coreChromVariables)
export(corePeaksVariables)
export(fillCoreChromVariables)
export(matchRtime)
export(plotChromatograms)
export(plotChromatogramsOverlay)
export(validChromData)
export(validPeaksData)
exportClasses(ChromBackend)
exportClasses(ChromBackendMemory)
exportClasses(ChromBackendMzR)
exportClasses(ChromBackendSpectra)
exportClasses(Chromatograms)
exportMethods("$")
exportMethods("$<-")
exportMethods("[")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("chromData<-")
exportMethods("chromIndex<-")
exportMethods("collisionEnergy<-")
exportMethods("dataOrigin<-")
exportMethods("intensity<-")
exportMethods("msLevel<-")
exportMethods("mz<-")
exportMethods("mzMax<-")
exportMethods("mzMin<-")
exportMethods("peaksData<-")
exportMethods("precursorMz<-")
exportMethods("precursorMzMax<-")
exportMethods("precursorMzMin<-")
exportMethods("processingChunkSize<-")
exportMethods("productMz<-")
exportMethods("productMzMax<-")
exportMethods("productMzMin<-")
exportMethods("rtime<-")
exportMethods(Chromatograms)
exportMethods(applyProcessing)
exportMethods(backendBpparam)
exportMethods(backendInitialize)
exportMethods(backendMerge)
exportMethods(backendParallelFactor)
exportMethods(c)
exportMethods(chromData)
exportMethods(chromExtract)
exportMethods(chromIndex)
exportMethods(chromVariables)
exportMethods(collisionEnergy)
exportMethods(compareChromatograms)
exportMethods(dataOrigin)
exportMethods(extractByIndex)
exportMethods(factorize)
exportMethods(filterChromData)
exportMethods(filterEmptyChromatograms)
exportMethods(filterPeaksData)
exportMethods(imputePeaksData)
exportMethods(intensity)
exportMethods(isEmpty)
exportMethods(isReadOnly)
exportMethods(length)
exportMethods(lengths)
exportMethods(msLevel)
exportMethods(mz)
exportMethods(mzMax)
exportMethods(mzMin)
exportMethods(peakBoundary)
exportMethods(peaksData)
exportMethods(precursorMz)
exportMethods(precursorMzMax)
exportMethods(precursorMzMin)
exportMethods(processingChunkFactor)
exportMethods(processingChunkSize)
exportMethods(productMz)
exportMethods(productMzMax)
exportMethods(productMzMin)
exportMethods(reset)
exportMethods(rtime)
exportMethods(setBackend)
exportMethods(show)
exportMethods(split)
exportMethods(supportsSetBackend)
importClassesFrom(ProtGenerics,ProcessingStep)
importClassesFrom(Spectra,Spectra)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(MsCoreUtils,between)
importFrom(MsCoreUtils,i2index)
importFrom(MsCoreUtils,maxi)
importFrom(MsCoreUtils,rbindFill)
importFrom(MsCoreUtils,reduce)
importFrom(MsCoreUtils,sumi)
importFrom(MsCoreUtils,valleys)
importFrom(MsCoreUtils,vapply1c)
importFrom(MsCoreUtils,vapply1l)
importFrom(ProtGenerics,ProcessingStep)
importFrom(Spectra,Spectra)
importFrom(Spectra,concatenateSpectra)
importFrom(Spectra,filterRanges)
importFrom(Spectra,filterRt)
importFrom(Spectra,peaksData)
importFrom(Spectra,rtime)
importFrom(Spectra,spectraData)
importFrom(Spectra,spectraVariables)
importFrom(data.table,rbindlist)
importFrom(grDevices,dev.flush)
importFrom(grDevices,dev.hold)
importFrom(grDevices,n2mfrow)
importFrom(grDevices,xy.coords)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,plot.xy)
importFrom(graphics,title)
importFrom(methods,"slot<-")
importFrom(methods,.valueClassTest)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,existsMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(stats,approx)
importFrom(stats,cor)
importFrom(stats,dnorm)
importFrom(stats,filter)
importFrom(stats,loess)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,spline)
importFrom(utils,capture.output)
importFrom(utils,head)
importMethodsFrom(ProtGenerics,"collisionEnergy<-")
importMethodsFrom(ProtGenerics,"dataOrigin<-")
importMethodsFrom(ProtGenerics,"intensity<-")
importMethodsFrom(ProtGenerics,"msLevel<-")
importMethodsFrom(ProtGenerics,"mz<-")
importMethodsFrom(ProtGenerics,"peaksData<-")
importMethodsFrom(ProtGenerics,"precursorMz<-")
importMethodsFrom(ProtGenerics,"processingChunkSize<-")
importMethodsFrom(ProtGenerics,"productMz<-")
importMethodsFrom(ProtGenerics,"rtime<-")
importMethodsFrom(ProtGenerics,addProcessing)
importMethodsFrom(ProtGenerics,applyProcessing)
importMethodsFrom(ProtGenerics,backendBpparam)
importMethodsFrom(ProtGenerics,backendInitialize)
importMethodsFrom(ProtGenerics,backendMerge)
importMethodsFrom(ProtGenerics,backendParallelFactor)
importMethodsFrom(ProtGenerics,collisionEnergy)
importMethodsFrom(ProtGenerics,dataOrigin)
importMethodsFrom(ProtGenerics,extractByIndex)
importMethodsFrom(ProtGenerics,intensity)
importMethodsFrom(ProtGenerics,isReadOnly)
importMethodsFrom(ProtGenerics,msLevel)
importMethodsFrom(ProtGenerics,mz)
importMethodsFrom(ProtGenerics,peaksData)
importMethodsFrom(ProtGenerics,peaksVariables)
importMethodsFrom(ProtGenerics,precursorMz)
importMethodsFrom(ProtGenerics,processingChunkFactor)
importMethodsFrom(ProtGenerics,processingChunkSize)
importMethodsFrom(ProtGenerics,productMz)
importMethodsFrom(ProtGenerics,rtime)
importMethodsFrom(ProtGenerics,setBackend)
importMethodsFrom(ProtGenerics,supportsSetBackend)
importMethodsFrom(S4Vectors,"$")
importMethodsFrom(S4Vectors,"$<-")
importMethodsFrom(S4Vectors,"[")
importMethodsFrom(S4Vectors,"[<-")
importMethodsFrom(S4Vectors,"[[")
importMethodsFrom(S4Vectors,"[[<-")
importMethodsFrom(S4Vectors,isEmpty)
importMethodsFrom(S4Vectors,split)
importMethodsFrom(methods,show)
